@@ -23,13 +23,14 @@ def get_text(nodelist):
     return rc
 
 VERSION_BODY = '''
-# This is the version of this tree, as created by %s from the Darcs patch
+# This is the version of this tree, as created by %(versiontool)s from the darcs patch
 # information: the main version number is taken from the most recent release
 # tag. If some patches have been added since the last release, this will have a
 # -NN "build number" suffix, or else a -rNN "revision number" suffix. Please see
 # pyutil.version_class for a description of what the different fields mean.
 
-verstr = "%s"
+__pkgname__ = "%(pkgname)s"
+verstr = "%(appversion)s"
 try:
     from pyutil.version_class import Version as pyutil_Version
     __version__ = pyutil_Version(verstr)
@@ -39,9 +40,13 @@ except (ImportError, ValueError):
     __version__ = distutils_Version(verstr)
 '''
 
-def write_version_py(verstr, outfname, EXE_NAME):
+def write_version_py(verstr, outfname, EXE_NAME, version_body, pkgname):
     f = open(outfname, "wt+")
-    f.write(VERSION_BODY % (EXE_NAME, verstr,))
+    f.write(version_body % {
+            'versiontool': EXE_NAME,
+            'appversion': verstr,
+            'pkgname': pkgname,
+            })
     f.close()
 
 def read_version_py(infname):
@@ -55,7 +60,7 @@ def read_version_py(infname):
         if mo:
             return mo.group(1)
 
-def update(pkgname, verfilename, revision_number=False, loud=False, abort_if_snapshot=False, EXE_NAME="darcsver"):
+def update(pkgname, verfilename, revision_number=False, loud=False, abort_if_snapshot=False, EXE_NAME="darcsver", version_body=VERSION_BODY):
     """
     @param revision_number If true, count the total number of patches in all
     history.  If false, count the total number of patches since the most recent
@@ -63,14 +68,21 @@ def update(pkgname, verfilename, revision_number=False, loud=False, abort_if_sna
 
     Returns a tuple of (exit code, new version string).
     """
+    if isinstance(verfilename, basestring):
+        verfilenames = [verfilename]
+    else:
+        verfilenames = verfilename
+    if isinstance(version_body, basestring):
+        verbodies = [version_body]
+    else:
+        verbodies = version_body
     rc = -1
-    cmd = ["changes", "--xml-output"]
-    if not revision_number:
-        cmd.append("--from-tag=^%s" % (pkgname,))
 
-    errput = None
+    # First we try "darcs query repo" because if that fails then we
+    # won't try "darcs changes" at all, because "darcs changes" emits
+    # an ugly error message when run in not-a-repo.
     try:
-        p = subprocess.Popen(["darcs"] + cmd, stdout=PIPE, stderr=PIPE, universal_newlines=True)
+        p = subprocess.Popen(["darcs", 'query', 'repo'], stdout=PIPE, stderr=PIPE, universal_newlines=True)
     except OSError, ose:
         if ose.errno == 2 and '~' in os.environ['PATH']:
             expanded_path = os.environ['PATH'].replace('~', os.path.expanduser('~'))
@@ -82,14 +94,27 @@ def update(pkgname, verfilename, revision_number=False, loud=False, abort_if_sna
     else:
         (output, errput) = p.communicate()
         rc = p.returncode
-    if rc != 0:
-        if errput:
-            log.info("%s: darcs wrote to stderr: '%s'" % (EXE_NAME, errput,))
-        if os.path.exists(verfilename):
-            log.info("%s: didn't version tags with 'darcs changes', and %s already exists, so leaving it alone." % (EXE_NAME, verfilename,))
-            return (0, read_version_py(verfilename))
+
+    if rc == 0:
+        cmd = ["changes", "--xml-output"]
+        if not revision_number:
+            cmd.append("--from-tag=^%s" % (pkgname,))
+        errput = None
+        try:
+            p = subprocess.Popen(["darcs"] + cmd, stdout=PIPE, stderr=PIPE, universal_newlines=True)
+        except OSError:
+            pass
         else:
-            log.warn("%s: didn't find version tags with 'darcs changes', and %s doesn't exist." % (EXE_NAME, verfilename))
+            (output, errput) = p.communicate()
+            rc = p.returncode
+            if rc != 0 and errput:
+                log.info("%s: darcs wrote to stderr: '%s'" % (EXE_NAME, errput,))
+    else:
+        if all([os.path.exists(vfn) for vfn in verfilenames]):
+            log.info("%s: using extant version file %s" % (EXE_NAME, verfilenames))
+            return (0, read_version_py(verfilenames[0]))
+        else:
+            log.warn("%s: didn't find version tags with darcs, and %s don't exist." % (EXE_NAME, verfilenames))
         return (rc, None)
 
     # Filter out bad chars that can cause the XML parser to give up in despair.
@@ -133,11 +158,11 @@ def update(pkgname, verfilename, revision_number=False, loud=False, abort_if_sna
     if not last_tag:
         if errput:
             log.info("%s: darcs wrote to stderr: '%s'" % (EXE_NAME, errput,))
-        if os.path.exists(verfilename):
-            log.warn("%s: I'm unable to find a tag in the darcs history matching \"%s\", so I'm leaving %s alone." % (EXE_NAME, regexstr, verfilename,))
-            return (0, read_version_py(verfilename))
+        if all([os.path.exists(vfn) for vfn in verfilenames]):
+            log.warn("%s: I'm unable to find a tag in the darcs history matching \"%s\", so I'm leaving %s alone." % (EXE_NAME, regexstr, verfilenames,))
+            return (0, read_version_py(verfilenames[0]))
         else:
-            log.warn("%s: I'm unable to find a tag in the darcs history matching \"%s\", and %s doesn't exist." % (EXE_NAME, regexstr, verfilename,))
+            log.warn("%s: I'm unable to find a tag in the darcs history matching \"%s\", and %s don't exist." % (EXE_NAME, regexstr, verfilenames,))
             return (0, None)
 
     if revision_number:
@@ -155,6 +180,7 @@ def update(pkgname, verfilename, revision_number=False, loud=False, abort_if_sna
             # this is a release
             verstr = last_tag
 
-    write_version_py(verstr, verfilename, EXE_NAME)
-    log.info("%s: wrote '%s' into %s" % (EXE_NAME, verstr, verfilename,))
+    for verfn, verbod in zip(verfilenames, verbodies):
+        write_version_py(verstr, verfn, EXE_NAME, verbod, pkgname)
+        log.info("%s: wrote '%s' into %s" % (EXE_NAME, verstr, verfn,))
     return (0, verstr)
